<?php
	
	list ($params) = $irc_call_args;
	
	if (!isset ($this->custom_vars->yandere_games))
	{
		$this->custom_vars->yandere_games = array ();
	}
	
	clearstatcache();
	
	if (!isset ($this->custom_vars->yandere_last_roles_set_up) || $this->custom_vars->yandere_last_roles_set_up < filemtime(dirname(__FILE__).'/set-up-roles.php'))
	{
		call('yandere/set-up-roles');
		$this->custom_vars->yandere_last_roles_set_up = time();
	}
	
	$user_id = call('yandere/get-user-id', $from);
	
	if (is_null($user_id))
	{
		output($to, $from.': For some reason, I don\'t know your hostmask. This is UNPOSSIBLE and means I have a bug. And you can\'t play until it\'s fixed. :(');
		return;
	}
	
	switch (count($params))
	{
	case 0:
		call('yandere/join', $user_id, array ());
		return;
	
	case 1:
		switch (strtolower($params[0]))
		{
		case 'abstain':
			call('yandere/action', 'abstain', $user_id, false);
			return;
		
		case 'end':
		case 'exit':
		case 'quit':
		case 'stop':
			$channel = irc_lower($to);
			
			if (!isset ($this->custom_vars->yandere_games[$channel]))
			{
				output($to, $from.': There\'s no game here.');
				return;
			}
			
			$game = $this->custom_vars->yandere_games[$channel];
			
			if ($game->starter != $user_id && !$this->is_admin($from))
			{
				output($to, $from.': You don\'t have permission to do that.');
				return;
			}
			
			output($to, $game->s.'The yandere game has been forced to end.'.$game->e);
			
			call('yandere/end-game', $channel);
			
			return;
		
		case 'extend':
			$channel = irc_lower($to);
			
			if (!isset ($this->custom_vars->yandere_games[$channel]))
			{
				output($to, $from.': There\'s no game here.');
				return;
			}
			
			$game = $this->custom_vars->yandere_games[$channel];
			
			$game->max_extensions++;
			
			output($to, 'Added another waiting extension.');
			return;
		
		case 'faster':
		case 'hurry':
		case 'next':
			$channel = irc_lower($to);
			
			if (!isset ($this->custom_vars->yandere_games[$channel]))
			{
				output($to, 'There\'s no game here.');
				return;
			}
			
			$game = $this->custom_vars->yandere_games[$channel];
			
			if (!isset ($game->players[$user_id]))
			{
				// User isn't in the game.
				
				notice($from, 'You aren\'t in this game.');
				return;
			}
			
			if (!$game->active)
			{
				if (count($game->players) < 4)
				{
					output($to, 'There aren\'t enough players yet.');
					return;
				}
				
				call('yandere/start-game', $channel, 0);
				return;
			}
			
			if ($game->turn % 2 == 0)
			{
				if (count($game->votes) == count($game->players))
				{
					// Everyone has voted. Move on immediately.
					
					call('yandere/night', $channel, call('yandere/solve-day-vote', $channel));
				}
				else
				{
					// Not everyone has voted. Wait a bit.
					
					output($channel, $game->s.'People are getting impatient! You all have 1 minute to cast your vote before the day ends.'.$game->e);
					
					$this->schedule_event(time() + 60, array ('yandere/day-timeout', $channel, $game->start_time, $game->turn));
				}
			}
			else
			{
				output($channel, $game->s.'People are getting impatient! The night roles have 1 minute to make their decisions before the night ends.'.$game->e);
				
				$this->schedule_event(time() + 60, array ('yandere/night-timeout', $channel, $game->start_time, $game->turn));
			}
			
			return;
		
		case 'help':
			$channel = irc_lower($to);
			
			$gameplay = 'This is a detective game. It cycles between two phases: During the day, everyone gets together and tries to figure out who they think is a yandere, and they can vote to lynch someone. During the night, players with special roles can perform actions that may shed some light.';
			
			$rules = 'Each night, the yanderes can pick one player to kill. The object of the game is to find and kill the yanderes before they kill you. PM conversations between players is allowed at any time, but you MAY NOT copy/paste PM messages into the channel.';
			
			if (!isset ($this->custom_vars->yandere_games[$channel]))
			{
				output($to, 'Yandere is a variation of the classic Mafia game, also known as Werewolf. Type "!yandere join" to start a game.');
				
				output($to, $gameplay);
				output($to, $rules);
				return;
			}
			
			$game = $this->custom_vars->yandere_games[$channel];
			
			if (!$game->active)
			{
				output($to, 'Yandere is a variation of the classic Mafia game, also known as Werewolf. A new game is about to start. Type "!yandere join" to join.');
				
				output($to, $gameplay);
				output($to, $rules);
			}
			elseif (!isset ($game->players[$user_id]))
			{
				notice($from, 'Please wait until the game is over and then type "!yandere join" to play.');
				
				notice($from, $gameplay);
				notice($from, $rules);
			}
			else
			{
				$role = $game->players[$user_id]->role;
				
				notice($from, 'You\'re a '.$this->custom_vars->yandere_role_singulars[$role].'. '.$this->custom_vars->yandere_role_descriptions[$role]);
				
				notice($from, $gameplay);
				notice($from, $rules);
			}
			return;
		}
		break;
	
	case 2:
		$command = strtolower($params[0]);
		
		switch ($command)
		{
		case 'assassinate':
		case 'attack':
		case 'battle':
		case 'destroy':
		case 'fight':
		case 'kill':
		case 'murder':
		case 'rape':
		case 'slaughter':
			call('yandere/action', 'kill', $user_id, $params[1]);
			return;
		
		case 'defend':
		case 'guard':
		case 'protect':
			call('yandere/action', 'guard', $user_id, $params[1]);
			return;
		
		case 'examine':
		case 'google':
		case 'infiltrate':
		case 'look':
		case 'snoop':
		case 'spy':
			call('yandere/action', 'spy', $user_id, $params[1]);
			return;
		
		case 'help':
			call('yandere/help', $params[1]);
			return;
		
		case 'vote':
			call('yandere/action', 'vote', $user_id, $params[1]);
			return;
		}
		break;
	}
	
	switch (strtolower($params[0]))
	{
	case 'begin':
	case 'join':
	case 'start':
		call('yandere/join', $user_id, array_slice($params, 1));
		return;
	
	default:
		if (count($params) == 1)
		{
			if ($to == $from)
			{
				$channel = call('yandere/get-player-channel', $user_id);
				
				if (is_null($channel))
				{
					// User isn't playing a game on this network.
					
					break;
				}
			}
			else
			{
				$channel = irc_lower($to);
				
				if (!isset ($this->custom_vars->yandere_games[$channel]))
				{
					// There is no game in this channel.
					
					break;
				}
			}
			
			$game = $this->custom_vars->yandere_games[$channel];
			
			if (!isset ($game->players[$user_id]))
			{
				// The user isn't in this game.
				
				break;
			}
			
			if ($to == $from && !is_null($game->players[$user_id]->action))
			{
				$action = $game->players[$user_id]->action;
			}
			else
			{
				$action = 'vote';
			}
			
			call('yandere/action', $action, $user_id, $params[0]);
			return;
		}
	}
	
	if ($to == $from)
	{
		notice($to, $from.': I don\'t recognize that command.');
	}
	else
	{
		output($to, $from.': I don\'t recognize that command.');
	}
	
?>
