<?php
	
	list ($channel, $start_time, $extensions) = $irc_call_args;
	
	if (!isset ($this->custom_vars->yandere_games[$channel]))
	{
		return;
	}
	
	$game = $this->custom_vars->yandere_games[$channel];
	
	if ($start_time != $game->start_time)
	{
		// This message is no longer relevant.
		return;
	}
	
	if ($game->active)
	{
		// The game already started.
		return;
	}
	
	if (count($game->players) < 4)
	{
		if ($extensions < $game->max_extensions)
		{
			output($channel, $game->s.'We don\'t have enough players yet. We need at least 4 players to start a game. I\'ll wait for another 60 seconds. Please type "!yandere" to join.'.$game->e);
			
			$this->schedule_event(time() + 60, array ('yandere/start-game', $channel, $game->start_time, $extensions + 1));
		}
		else
		{
			output($channel, $game->s.'There aren\'t enough players to start a yandere game. Try again later.'.$game->e);
			
			unset ($this->custom_vars->yandere_games[$channel]);
		}
		
		return;
	}
	
	if ($game->restrict_voice)
	{
		// Silence all users in the channel.
		
		$this->send('MODE '.$channel.' +m');
	}
	
	call('yandere/remove-voice', $channel);
	
	$game->active = true;
	
	// Get a random-order list of the players, to simplify random role assignment.
	
	$user_ids = array_keys($game->players);
	shuffle($user_ids);
	
	// Find out how many of each role we're going to have.
	
	$count = count($user_ids);
	
	$units = array ();
	
	$units[$this->custom_vars->yandere_standard_roles['mafia']] = floor($count / 4);
	$units[$this->custom_vars->yandere_standard_roles['detectives']] = floor($count / 14) + 1;
	
	foreach ($units as $key => $value)
	{
		if ($value == 0)
		{
			unset ($units[$key]);
		}
	}
	
	$remaining = $count - array_sum($units);
	
	switch ($game->complexity)
	{
	case 'basic':
		$units[$this->custom_vars->yandere_standard_roles['innocents']] = $remaining;
		break;
	
	case 'normal':
		for ($i=0; $i < $remaining; $i++)
		{
			$units[array_rand_weighted($this->custom_vars->yandere_role_odds)]++;
		}
		break;
	}
	
	// Assign the roles.
	
	$unit_list = array ();
	
	$yandere_players = array ();
	
	$i = 0;
	
	foreach ($units as $role => $num)
	{
		$action = isset ($this->custom_vars->yandere_role_actions[$role]) ? $this->custom_vars->yandere_role_actions[$role] : null;
		
		if (!$game->blind_roles || $action == 'vote')
		{
			$unit_list[] = $num.' '.str_plural($this->custom_vars->yandere_role_singulars[$role], $num, $role);
		}
		
		for ($j=0; $j < $num; $j++)
		{
			$game->players[$user_ids[$i]]->role = $role;
			
			if ($action == 'vote')
			{
				$yandere_players[] = $game->players[$user_ids[$i]]->nick;
			}
			
			$i++;
		}
	}
	
	$game->init_players = $game->players;
	
	foreach ($game->players as $player)
	{
		notice($player->nick, $game->s.'You\'re a '.$this->custom_vars->yandere_role_singulars[$player->role].'. '.$this->custom_vars->yandere_role_descriptions[$player->role].$game->e);
		
		if ($player->action == 'vote')
		{
			notice($player->nick, $game->s.'The yanderes in this game are: '.implode_comma_series($yandere_players).'.'.$game->e);
		}
	}
	
	// Introduction message
	
	$description1 = $this->custom_vars->yandere_murder_descriptions[mt_rand(0, count($this->custom_vars->yandere_murder_descriptions) - 1)];
	
	$description1 .= ' '.$this->custom_vars->yandere_weapons[mt_rand(0, count($this->custom_vars->yandere_weapons) - 1)].'. ';
	
	switch (mt_rand(0, 2))
	{
	case 0:
		$description2 = 'This looks like the work of a yandere!';
		break;
	
	case 1:
		$description2 = 'There\'s only one explanation: there is a yandere among us!';
		break;
	
	case 2:
		$description2 = 'She seems to have been the victim of a yandere!';
		break;
	}
	
	switch (mt_rand(0, 8))
	{
	case 0:
		output($channel, $game->s.'TAIHEN!! The loli village wakes up one morning to a horrible surprise: Shin-chan '.$description1.$description2.$game->e);
		break;
	
	case 1:
		output($channel, $game->s.'All the little lolis go to school one day and notice that Shin-chan seems to be missing. After school, you go to her house to check on her, and find that she '.$description1.$description2.$game->e);
		break;
	
	case 2:
		output($channel, $game->s.'At '.mt_rand(2, 4).':'.str_pad(mt_rand(0, 59), 2, '0', STR_PAD_LEFT).' in the morning, the neighborhood lolis wake up to a scream. It sounds like Shin-chan! You all run to her room and find that she '.$description1.$description2.$game->e);
		break;
	
	case 3:
		output($channel, $game->s.'Today is Shin-chan\'s birthday! And she\'s having ichigo cake! All the townslolis eagerly come to her house, but they\'re met with a surprise: Shin-chan '.$description1.$description2.$game->e);
		break;
	
	case 4:
		output($channel, $game->s.'Shin-chan is a quiet, moe loli. But she\'s especially quiet today, because she '.$description1.$description2.$game->e);
		break;
	
	case 5:
		output($channel, $game->s.'It\'s Christmas morning! Shin-chan rushes into the living room and sees a great big present with her name on it. Full of excitement, she rips open the box and finds some sort of weird alarm clock attached to several red sticks. It proceeds to explode. '.$description2.$game->e);
		break;
	
	case 6:
		output($channel, $game->s.'Shin-chan is riding merrily on the bus on her way to school. Little does she know it\'s rigged with hundreds of explosives. The bus explodes in a mighty inferno, and Shin-chan bits go flying everywhere. '.$description2.$game->e);
		break;
	
	case 7:
		output($channel, $game->s.'All her life, Shin-chan had been plagued by an illness that confined her to her room. But today, she decided to fulfill her dream: to risk it and go to school like all the other lolis. But her lunch was poisoned and she died. '.$description2.$game->e);
		break;
	
	case 8:
		output($channel, $game->s.'Everybody loves Shin-chan, except for whoever set her on fire last night. Now Shin-chan is charcoal. '.$description2.$game->e);
		break;
	}
	
	rsort($unit_list);
	
	output($channel, $game->s.'This game has '.implode_comma_series($unit_list).'. It\'s your job to determine the '.str_plural('yandere', $units['yanderes'], 'yanderes').'.'.$game->e);
	
	call('yandere/night', $channel);
	
?>
